#include "comtypes.H"
#include "allglobals.H"
#include <stdlib.h>
#include <stdio.h>
#include <sys/param.h>
#include <ctype.h>
#include <netdb.h>

#define    IPPROTO_IP              0

#define         ICMP_UNREACH_NET        0               /* bad net */
#define         ICMP_UNREACH_HOST       1               /* bad host */
#define         ICMP_UNREACH_PROTOCOL   2               /* bad protocol */
#define         ICMP_UNREACH_PORT       3               /* bad port */
#define         ICMP_UNREACH_NEEDFRAG   4               /* IP_DF caused drop */
#define         ICMP_UNREACH_SRCFAIL    5               /* src route failed */
#define         ICMP_UNREACH_NET_UNKNOWN 6              /* unknown net */
#define         ICMP_UNREACH_HOST_UNKNOWN 7             /* unknown host */
#define         ICMP_UNREACH_ISOLATED   8               /* src host isolated */
#define         ICMP_UNREACH_NET_PROHIB 9               /* prohibited access */
#define         ICMP_UNREACH_HOST_PROHIB 10             /* ditto */
#define         ICMP_UNREACH_TOSNET     11              /* bad tos for net */
#define         ICMP_UNREACH_TOSHOST    12              /* bad tos for host */
#define         ICMP_UNREACH_FILTER_PROHIB 13           /* admin prohib */
#define         ICMP_UNREACH_HOST_PRECEDENCE 14         /* host prec vio. */
#define         ICMP_UNREACH_PRECEDENCE_CUTOFF 15       /* prec cutoff */

struct icmpcode {
	int	code;
	char	*str;
};

static struct icmpcode icmpcodes[] = {
      { ICMP_UNREACH_NET,		"net" },
      { ICMP_UNREACH_HOST,		"host" },
      { ICMP_UNREACH_PROTOCOL,		"protocol" },
      { ICMP_UNREACH_PORT,		"port" },
      { ICMP_UNREACH_NEEDFRAG,		"needfrag" },
      { ICMP_UNREACH_SRCFAIL,		"srcfail" },
      { ICMP_UNREACH_NET_UNKNOWN,	"net-unknown" },
      { ICMP_UNREACH_HOST_UNKNOWN,	"host-unknown" },
      { ICMP_UNREACH_ISOLATED,		"isolated" },
      { ICMP_UNREACH_NET_PROHIB,	"net-prohib" },
      { ICMP_UNREACH_HOST_PROHIB,	"host-prohib" },
      { ICMP_UNREACH_TOSNET,		"tosnet" },
      { ICMP_UNREACH_TOSHOST,		"toshost" },
      { ICMP_UNREACH_FILTER_PROHIB,	"filter-prohib" },
      { ICMP_UNREACH_HOST_PRECEDENCE,	"host-precedence" },
      { ICMP_UNREACH_PRECEDENCE_CUTOFF,	"precedence-cutoff" },
      { 0, NULL }
};

void make_error(char * str,FWRule * pRule)
{
	pRule->m_strError = str;
}

void add(int ac,char ** av,FWRule * pRule)
{
	u_char proto;
	protoent *pe;
	// TODO - zero all rule parameters
	pRule->ipfw_number = 0;
	pRule->ipfw_bprob = false;
	pRule->ipfw_matchprob = 1;
	pRule->ipfw_action = "";
	pRule->ipfw_blog = false;
	pRule->ipfw_protocol = "ip";
	pRule->ipfw_divert_port = 0;
	pRule->ipfw_divert_port_name = "";
	pRule->ipfw_unreach_code = 0;
	pRule->ipfw_fwd_address = "";
	pRule->ipfw_skipto_rule = 0;
	pRule->ipfw_blogamount = false;
	pRule->ipfw_logamount = 0;
	pRule->ipfw_not_src = false;
	pRule->ipfw_not_dst = false;
	pRule->ipfw_src_address = "127.0.0.1";
	pRule->ipfw_src_port = "";
	pRule->ipfw_dst_address = "127.0.0.1";
	pRule->ipfw_dst_port = "";
	pRule->ipfw_uid = "";
	pRule->ipfw_gid = "";
	pRule->ipfw_direction = "";
	pRule->ipfw_option_keepstate = false;
	pRule->ipfw_option_keepstate_method = "";
	pRule->ipfw_option_bridged = false;
	pRule->ipfw_option_fragment = false;
	pRule->ipfw_option_established = false;
	pRule->ipfw_option_setup = false;
	pRule->ipfw_option_tcpflags = false;
	pRule->ipfw_option_tcpflags_string = "";
	pRule->ipfw_option_ipoptions = false;
	pRule->ipfw_option_ipoptions_string = "";
	pRule->ipfw_via = "";
	pRule->ipfw_option_tcpoptions = false;
	pRule->ipfw_option_tcpoptions_string = "";
	pRule->ipfw_option_icmptypes = false;
	pRule->ipfw_option_icmptypes_string = "";		
	/* Rule number */
	if (ac && isdigit(**av)) {
		pRule->ipfw_number = atoi(*av); av++; ac--;
	}

	/* Action */
	if (ac > 1 && !strncmp(*av, "prob", strlen(*av) ) ) {
		pRule->ipfw_bprob = true;
		double d = strtod(av[1], NULL);
		if (d <= 0 || d > 1)
		{
			make_error("illegal match prob",pRule);
			return;
		}
		pRule->ipfw_matchprob = d;
		av += 2 ; ac -= 2 ;
	}

	if (ac == 0)
	{
		make_error("missing action",pRule);
		return;
	}
	if (!strncmp(*av,"accept",strlen(*av))
		    || !strncmp(*av,"pass",strlen(*av))
		    || !strncmp(*av,"allow",strlen(*av))
		    || !strncmp(*av,"permit",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
	} else if (!strncmp(*av,"count",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
        } else if (!strncmp(*av,"pipe",strlen(*av))) {
                pRule->ipfw_action = *av; av++; ac--;
                if (!ac)
                        make_error("missing pipe number",pRule);
                pRule->ipfw_divert_port = strtoul(*av, NULL, 0); av++; ac--;
        } else if (!strncmp(*av,"queue",strlen(*av))) {
                pRule->ipfw_action = *av; av++; ac--;
                if (!ac)
			 {
                    make_error("missing queue number",pRule);
				return;				
			 }
                pRule->ipfw_divert_port = strtoul(*av, NULL, 0); av++; ac--;
	} else if (!strncmp(*av,"divert",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
		if (!ac)
		{
			make_error("missing divert port",pRule);
			return;
		}
		pRule->ipfw_divert_port = strtoul(*av, NULL, 0); av++; ac--;
		if (pRule->ipfw_divert_port == 0) {
			struct servent *s;
			setservent(1);
			s = getservbyname(av[-1], "divert");
			if (s != NULL)
			{
				pRule->ipfw_divert_port = ntohs(s->s_port);
				pRule->ipfw_divert_port_name = av[-1];
			}
			else
			{
				make_error("illegal divert port",pRule);
				return;
			}
		}
	} else if (!strncmp(*av,"tee",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
		if (!ac)
		{
			make_error("missing tee divert port",pRule);
			return;
		}
		pRule->ipfw_divert_port = strtoul(*av, NULL, 0); av++; ac--;
		if (pRule->ipfw_divert_port == 0) {
			struct servent *s;
			setservent(1);
			s = getservbyname(av[-1], "divert");
			if (s != NULL)
			{
				pRule->ipfw_divert_port = ntohs(s->s_port);
				pRule->ipfw_divert_port_name = av[-1];
			}
			else
			{
				make_error("illegal tee divert port",pRule);
				return;
			}
		}
	} else if (!strncmp(*av,"fwd",strlen(*av)) ||
		   !strncmp(*av,"forward",strlen(*av))) {
		
		pRule->ipfw_action = *av; av++; ac--;
		if (!ac)
		{
			make_error("missing forwarding IP address",pRule);
			return;
		}
		if (!strchr(*av,'.'))
		{
			make_error("wrong forwarding IP address",pRule);
			return;
		}
		pRule->ipfw_fwd_address = *av; av++; ac--;

	} else if (!strncmp(*av,"skipto",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
		if (!ac)
		{
			make_error("missing skipto rule number",pRule);
			return;
		}
		pRule->ipfw_skipto_rule = strtoul(*av, NULL, 0); av++; ac--;
	} else if ((!strncmp(*av,"deny",strlen(*av))
		    || !strncmp(*av,"drop",strlen(*av)))) {
		pRule->ipfw_action = *av; av++; ac--;
	} else if (!strncmp(*av,"reject",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
	} else if (!strncmp(*av,"reset",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
	} else if (!strncmp(*av,"unreach",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
		u_long val;
		char *s;
		icmpcode *ic;
		val = strtoul(*av, &s, 0);
		if (s != *av && *s == '\0' && val < 0x100) {
			pRule->ipfw_unreach_code = val;
		}
		else
		for (ic = icmpcodes; ic->str; ic++)
			if (!strcasecmp(*av, ic->str)) {
				pRule->ipfw_unreach_code = ic->code;
				break;
			};
		if (!ic->str)
		{
			make_error("unknown ICMP unreachable code",pRule);
			return;
		}
		av++; ac--;
	} else if (!strncmp(*av,"check-state",strlen(*av))) {
		pRule->ipfw_action = *av; av++; ac--;
		goto done ;
	} else {
		make_error("invalid action", pRule);
		return;
	}

	/* [log] */
	if (ac && !strncmp(*av,"log",strlen(*av))) {
		pRule->ipfw_blog = true; av++; ac--;
	}
	if (ac && !strncmp(*av,"logamount",strlen(*av))) {
		pRule->ipfw_blogamount = true;
		ac--; av++;
		if (!ac)
		{
			make_error("logamount requires argument",pRule);
			return;
		}
		pRule->ipfw_logamount = atoi(*av);
		if (pRule->ipfw_logamount < 0)
		{
			make_error("logamount argument must be positive",pRule);
			return;
		}
		ac--; av++;
	}

	/* protocol */
	if (ac == 0)
	{
		make_error("missing protocol",pRule);
		return;
	}
	if ((proto = atoi(*av)) > 0) {
		pRule->ipfw_protocol = *av; av++; ac--;
	} else if (!strncmp(*av,"all",strlen(*av))) {
		pRule->ipfw_protocol = "all"; av++; ac--;
	} else if ((pe = getprotobyname(*av)) != NULL) {
		pRule->ipfw_protocol = *av; av++; ac--;
	} else {
		make_error("invalid protocol",pRule);
		return;
	}

	/* from */
	if (ac && !strncmp(*av,"from",strlen(*av))) { av++; ac--; }
	else
	{
		make_error("missing from",pRule);
		return;
	}

	if (ac && !strncmp(*av,"not",strlen(*av))) {
		pRule->ipfw_not_src = true;
		av++; ac--;
	}
	if (!ac)
	{
		make_error("missing arguments",pRule);
		return;
	}

	pRule->ipfw_src_address = *av;
	av++;ac--;
	if (ac && strncmp(*av,"to",strlen(*av))) {
		pRule->ipfw_src_port = *av;
		av++; ac--;
	}

	/* to */
	if (ac && !strncmp(*av,"to",strlen(*av))) { av++; ac--; }
	else
	{
		make_error("missing to",pRule);
		return;
	}
	if (ac && !strncmp(*av,"not",strlen(*av))) {
		pRule->ipfw_not_dst = true;
		av++; ac--;
	}
	if (!ac)
	{
		make_error("missing arguments",pRule);
		return;
	}
	pRule->ipfw_dst_address = *av;
	av++;ac--;
	if (ac && strncmp(*av,"via",strlen(*av)) &&
		strncmp(*av,"uid",strlen(*av)) &&
		strncmp(*av,"gid",strlen(*av)) &&
		strncmp(*av,"established",strlen(*av)) &&
		strncmp(*av,"setup",strlen(*av)) &&
		strncmp(*av,"in",strlen(*av)) &&
		strncmp(*av,"out",strlen(*av)) &&
		strncmp(*av,"ipoptions",strlen(*av)) &&
		strncmp(*av,"tcpoptions",strlen(*av)) &&
		strncmp(*av,"recv",strlen(*av)) &&
		strncmp(*av,"xmit",strlen(*av)) &&
		strncmp(*av,"tcpflags",strlen(*av)) &&
		strncmp(*av,"icmptypes",strlen(*av)) &&
		strncmp(*av,"keep-state",strlen(*av)) &&
		strncmp(*av,"bridged",strlen(*av)) &&
		strncmp(*av,"frag",strlen(*av))
		) {
		pRule->ipfw_dst_port = *av;
		av++; ac--;
	}



	while (ac) {
		if (!strncmp(*av,"uid",strlen(*av))) {
			ac--; av++;
			if (!ac)
			{
				make_error("uid requires argument",pRule);
				return;
			}
			pRule->ipfw_uid = *av;
			ac--; av++;
			continue;
		}
		if (!strncmp(*av,"gid",strlen(*av))) {
			ac--; av++;
			if (!ac)
			{
				make_error("gid requires argument",pRule);
				return;
			}
			
			pRule->ipfw_gid = *av;
			ac--; av++;
			continue;
		}
		if (!strncmp(*av,"in",strlen(*av))) { 
			pRule->ipfw_direction = "in";
			av++; ac--; continue;
		}
                if (!strncmp(*av,"keep-state",strlen(*av))) { 
                        unsigned long type ;
                        pRule->ipfw_option_keepstate = true; 
                        av++; ac--;
                        if (ac > 0 && (type = atoi(*av)) != 0) {
                            pRule->ipfw_option_keepstate_method = *av;
                            av++; ac--;
                        }
                        continue;
                }
                if (!strncmp(*av,"bridged",strlen(*av))) { 
                        pRule->ipfw_option_bridged = true;
                        av++; ac--; continue;
                }
		if (!strncmp(*av,"out",strlen(*av))) { 
			pRule->ipfw_direction = "out";
			av++; ac--; continue;
		}
		
		if ((ac && !strncmp(*av,"via",strlen(*av)))||
			(ac && !strncmp(*av,"xmit",strlen(*av)))||
			 (ac && !strncmp(*av,"recv",strlen(*av)))){
			av++; ac--; 
			pRule->ipfw_via = *av;
			av++; ac--; continue;
		}
		if (!strncmp(*av,"fragment",strlen(*av))) {
			pRule->ipfw_option_fragment = true;
			av++; ac--; continue;
		}
		if (!strncmp(*av,"ipoptions",strlen(*av))) {
			pRule->ipfw_option_ipoptions = true;
			av++; ac--; 
			if (!ac)
			{
				make_error("missing argument for ipoptions",pRule);
				return;
			}
			pRule->ipfw_option_ipoptions_string = *av;
			av++; ac--; continue;
		}
		if (pRule->ipfw_protocol == "tcp") {
			if (!strncmp(*av,"established",strlen(*av))) { 
				pRule->ipfw_option_established = true;
				av++; ac--; continue;
			}
			if (!strncmp(*av,"setup",strlen(*av))) { 
				pRule->ipfw_option_setup = true;
				av++; ac--; continue;
			}
			if (!strncmp(*av,"tcpflags",strlen(*av)) || !strncmp(*av,"tcpflgs",strlen(*av))) { 
				av++; ac--;
				pRule->ipfw_option_tcpflags = true; 
				if (!ac)
				{
					make_error("missing argument for tcpflags",pRule);
					return;
				}
				pRule->ipfw_option_tcpflags_string = *av;
				av++; ac--; continue;
			}
			if (!strncmp(*av,"tcpoptions",strlen(*av)) || !strncmp(*av, "tcpopts",strlen(*av))) { 
				av++; ac--;
				pRule->ipfw_option_tcpoptions = true;
				if (!ac)
				{
					make_error("missing argument for tcpoptions",pRule);
					return;
				}
				pRule->ipfw_option_tcpoptions_string = *av;
				av++; ac--; continue;
			}
		}
		if (pRule->ipfw_protocol == "icmp") {
			if (!strncmp(*av,"icmptypes",strlen(*av))) {
				av++; ac--;
				pRule->ipfw_option_icmptypes = true;
				if (!ac)
				{
					make_error("missing argument for icmptypes",pRule);
					return;
				}
				pRule->ipfw_option_icmptypes_string = *av;
				av++; ac--; continue;
			}
		}
		make_error("unknown argument",pRule);
		return;
	}
done:
	return;
}

FWRule::FWRule()
{
	m_strError = "";
	ac = 0;
	av = NULL;
}

FWRule::~FWRule()
{
	clear();
}

void FWRule::dump(ostream & dumpstream)
{
	if (m_strError != "")
	{
		dumpstream <<"Error status: " <<m_strError <<endl;
		return;
	}

	dumpstream <<"No.: " <<ipfw_number <<endl;
	dumpstream <<"Prob: " <<(ipfw_bprob ? "yes" : "no") <<endl;
	dumpstream <<"Match prob: " <<ipfw_matchprob <<endl;
	dumpstream <<"Log: " <<(ipfw_blogamount ? "yes" : "no") <<endl;
	dumpstream <<"Logamount: " <<ipfw_logamount <<endl;
	dumpstream <<"Proto: " <<ipfw_protocol <<endl;
	dumpstream <<"Action: " <<ipfw_action <<endl;
	dumpstream <<"Forward address: " <<ipfw_fwd_address <<endl;
	dumpstream <<"Unreach code: " <<ipfw_unreach_code <<endl;
	dumpstream <<"Divert port: " <<ipfw_divert_port <<endl;
	dumpstream <<"Skip to rule: " <<ipfw_skipto_rule <<endl;
	dumpstream <<"From: ";
	dumpstream <<(ipfw_not_src ? "NOT" : "");
	dumpstream <<ipfw_src_address <<endl;
	dumpstream <<"From_port: " <<ipfw_src_port <<endl;
	dumpstream <<"To: ";
	dumpstream <<(ipfw_not_dst ? "NOT" : "");
	dumpstream <<ipfw_dst_address <<endl;
	dumpstream <<"To_port: " <<ipfw_dst_port <<endl;
	dumpstream <<"Via: " <<ipfw_via <<endl;
	dumpstream <<"Uid: " <<ipfw_uid <<endl;
	dumpstream <<"Gid: " <<ipfw_gid <<endl;
	dumpstream <<"Direction: " <<ipfw_direction <<endl;
	dumpstream <<"Keepstate: " <<(ipfw_option_keepstate ? "yes" : "no") <<endl;
	dumpstream <<"Bridged: " <<(ipfw_option_bridged ? "yes" : "no") <<endl;
	dumpstream <<"Fragment: " <<(ipfw_option_fragment ? "yes" : "no") <<endl;
	dumpstream <<"IP options: " <<(ipfw_option_ipoptions ? "yes" : "no") <<endl;
	dumpstream <<"IP options: " <<ipfw_option_ipoptions_string <<endl;
	dumpstream <<"TCP options: " <<(ipfw_option_tcpoptions ? "yes" : "no") <<endl;
	dumpstream <<"TCP options: " <<ipfw_option_tcpoptions_string <<endl;
	dumpstream <<"ICMP types: " <<(ipfw_option_icmptypes ? "yes" : "no") <<endl;
	dumpstream <<"IP options: " <<ipfw_option_icmptypes_string <<endl;
	dumpstream <<"Established: " <<(ipfw_option_established ? "yes" : "no") <<endl;
	dumpstream <<"Setup: " <<(ipfw_option_setup ? "yes" : "no") <<endl;
}

void FWRule::readString(char * pstr)
{
	QString str = pstr;
	str = str.stripWhiteSpace();
	splitArgs(str);
	add(ac,av,this);
}

void FWRule::splitArgs(QString & str)
{
	ac = 1;
	int nCurrentIndex = 0;
	int nPrevIndex = 0;
	/* First count spaces in the string */
	while ((nCurrentIndex = str.find(" ",nCurrentIndex)) != -1)
	{
		nCurrentIndex++;
		ac++;
	}
	av = new (char*)[ac];
	
	/* Now fill the av[] array */
	for (int i =0;i < ac;i++)
	{
		nCurrentIndex = str.find(" ",nPrevIndex)+1;
		av[i] = new char[strlen(str.mid(nPrevIndex,nCurrentIndex - nPrevIndex-1))+1];
		strcpy(av[i],str.mid(nPrevIndex,nCurrentIndex - nPrevIndex-1));
		nPrevIndex = nCurrentIndex;
	}
}

QString numtostr(int num)
{
	char str[100];
	sprintf(str,"%d",num);
	return str;
}

QString numtostr(double num)
{
	char str[100];
	sprintf(str,"%f",num);
	while (str[strlen(str)-1] == '0')
		str[strlen(str)-1] = '\0';
	return str;
}

QString FWRule::formRule()
{
	QString tmpStr = "";
	QString strRuleNumber = "";
	tmpStr+="add ";
	if (ipfw_number != 0)
	{
		strRuleNumber = numtostr(ipfw_number).rightJustify(5,'0');
//		strRuleNumber.rightJustify(5,'0');
		tmpStr+= strRuleNumber;
		tmpStr+=" ";
	}
	
	if (ipfw_bprob)
	{
		tmpStr+="prob "; tmpStr+=numtostr(ipfw_matchprob);
		tmpStr+=" ";
	}
	
	tmpStr+=ipfw_action; tmpStr+=" ";
	
	if (ipfw_action == "unreach")
	{
		tmpStr+=numtostr((int)ipfw_unreach_code);
		tmpStr+=" ";
	}
	
	if ((ipfw_action == "divert")||(ipfw_action == "pipe")||(ipfw_action == "queue"))
	{
		if (ipfw_divert_port !=0)
			tmpStr+=numtostr((int)ipfw_divert_port);
		else
			tmpStr+=ipfw_divert_port_name;
		tmpStr+=" ";		
	}
	
	if (ipfw_action == "tee")
	{
		if (ipfw_divert_port !=0)
			tmpStr+=numtostr((int)ipfw_divert_port);
		else
			tmpStr+=ipfw_divert_port_name;
		tmpStr+=" ";		
	}
	
	if ((ipfw_action == "fwd") || (ipfw_action == "forward"))
	{
		tmpStr+=ipfw_fwd_address;
		tmpStr+=" ";
	}
	if (ipfw_action == "skipto")
	{
		tmpStr+=numtostr((int)ipfw_skipto_rule);
		tmpStr+=" ";
	}	
	
	if (ipfw_blog)
	{
		tmpStr+="log ";
		if (ipfw_blogamount)
		{
			tmpStr+="logamount ";
			tmpStr+=numtostr((int)ipfw_logamount);
			tmpStr+=" ";
		}
	}
	
	tmpStr+=ipfw_protocol;
	tmpStr+=" ";
	
	tmpStr+="from ";
	if (ipfw_not_src)
		tmpStr+="not ";
	tmpStr+=ipfw_src_address;
	tmpStr+=" ";
	if (ipfw_src_port != "")
	{
		tmpStr+=ipfw_src_port;
		tmpStr+=" ";
	}
	
	tmpStr+="to ";
	if (ipfw_not_dst)
		tmpStr+="not ";
	tmpStr+=ipfw_dst_address;
	tmpStr+=" ";
	if (ipfw_dst_port != "")
	{
		tmpStr+=ipfw_dst_port;
		tmpStr+=" ";
	}
	
	if (ipfw_direction != "")
	{
		tmpStr+=ipfw_direction;
		tmpStr+=" ";
	}
	
	if (ipfw_via != "")
	{
		tmpStr+="via ";
		tmpStr+=ipfw_via;
		tmpStr+=" ";
	}
	
	if (ipfw_option_keepstate)
	{
		tmpStr+="keep-state ";
		if (ipfw_option_keepstate_method != "")
		{
			tmpStr+=ipfw_option_keepstate_method;
			tmpStr+=" ";
		}
	}
	if (ipfw_option_bridged)
		tmpStr+="bridged ";
	if (ipfw_option_fragment)
		tmpStr+="frag ";
	if (ipfw_option_setup)
		tmpStr+="setup ";
	if (ipfw_option_established)
		tmpStr+="established ";
	if (ipfw_option_ipoptions)
	{
		tmpStr+="ipoptions ";
		tmpStr+=ipfw_option_ipoptions_string;
		tmpStr+=" ";
	}
	
	if (ipfw_option_tcpoptions)
	{
		tmpStr+="tcpoptions ";
		tmpStr+=ipfw_option_tcpoptions_string;
		tmpStr+=" ";
	}
	
	if (ipfw_option_tcpflags)
	{
		tmpStr+="tcpflags ";
		tmpStr+=ipfw_option_tcpflags_string;
		tmpStr+=" ";
	}
	if (ipfw_option_icmptypes)
	{
		tmpStr+="icmptypes ";
		tmpStr+=ipfw_option_icmptypes_string;
		tmpStr+=" ";
	}
	
	if (ipfw_uid != "")
	{
		tmpStr+="uid ";
		tmpStr+=ipfw_uid;
		tmpStr+=" ";
	}
	
	if (ipfw_gid != "")
	{
		tmpStr+="gid ";
		tmpStr+=ipfw_gid;
		tmpStr+=" ";
	}
	return tmpStr.stripWhiteSpace();
}

void FWRule::clear()
{
	if (ac == 0) return;
	for (int index=0;index<ac;index++)
		delete[]av[index];
	delete []av;
	ac = 0;
	av = NULL;
}



