#include "comtypes.H"
#include "allglobals.H"
#include "kernelconfigtab.H"

#include <qapplication.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qcursor.h>
#include <qframe.h>
#include <stdlib.h>
#include <string.h>
#include <fstream.h>
QString numtostr(int num);
KernelConfigTab::KernelConfigTab( QWidget* parent,  const char* name, WFlags fl )
    : KernelConfigTabBase( parent, name, fl )
{
/*	QGridLayout * grid_top = new QGridLayout(this,10,5,4);
	QGridLayout * grid_buttons = new QGridLayout(Frame19,1,6,4);
	grid_top->addMultiCellWidget(TextLabel1,0,0,0,4);
	grid_top->addMultiCellWidget(checkbox_fw_enable,1,1,0,1);
	grid_top->addMultiCellWidget(checkbox_fw_verbose,2,2,0,1);
	grid_top->addMultiCellWidget(checkbox_fw_debug,3,3,0,1);
	grid_top->addWidget(TextLabel3,4,0);
	grid_top->addWidget(spinbox_fw_verbose_limit,4,1);
	grid_top->addWidget(TextLabel4,5,0);
	grid_top->addWidget(lineedit_fw_dyn_buckets,5,1);
	grid_top->addWidget(TextLabel5,6,0);
	grid_top->addWidget(lineedit_fw_curr_dyn_buckets,6,1);
	grid_top->addWidget(TextLabel7,7,0);
	grid_top->addWidget(lineedit_fw_dyn_count,7,1);
	grid_top->addWidget(TextLabel8,1,3);
	grid_top->addWidget(spinbox_fw_dyn_max,1,4);
	grid_top->addWidget(TextLabel9,2,3);
	grid_top->addWidget(spinbox_fw_ack_lifetime,2,4);
	grid_top->addWidget(TextLabel10,3,3);
	grid_top->addWidget(spinbox_fw_syn_lifetime,3,4);
	grid_top->addWidget(TextLabel11,4,3);
	grid_top->addWidget(spinbox_fw_fin_lifetime,4,4);
	grid_top->addWidget(TextLabel12,5,3);
	grid_top->addWidget(spinbox_fw_rst_lifetime,5,4);
	grid_top->addWidget(TextLabel13,6,3);
	grid_top->addWidget(spinbox_fw_short_lifetime,6,4);
	grid_top->addMultiCellWidget(Frame19,9,9,0,4);
	grid_buttons->addWidget(pushbutton_apply,0,0);
	grid_buttons->addWidget(pushbutton_revert,0,2);
	grid_buttons->addWidget(pushbutton_explain,0,5); */
	connect(pushbutton_apply,SIGNAL(clicked()),this,SLOT(onApply()));
	connect(pushbutton_revert,SIGNAL(clicked()),this,SLOT(onRevert()));
	lineedit_fw_dyn_buckets->setReadOnly(TRUE);
	lineedit_fw_curr_dyn_buckets->setReadOnly(TRUE);
	lineedit_fw_dyn_count->setReadOnly(TRUE);
	getSysctlValues();
	pushbutton_revert->setEnabled(FALSE);
	pushbutton_apply->setEnabled(FALSE);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
KernelConfigTab::~KernelConfigTab()
{
    // no need to delete child widgets, Qt does it all for us
}
void KernelConfigTab::onApply()
{
	qApp->setOverrideCursor(waitCursor);
	setSysctlValues();
	getSysctlValues();
	pushbutton_revert->setEnabled(FALSE);
	pushbutton_apply->setEnabled(FALSE);
	qApp->restoreOverrideCursor();
}

void KernelConfigTab::onRevert()
{	
	qApp->setOverrideCursor(waitCursor);
	getSysctlValues();
	pushbutton_revert->setEnabled(FALSE);
	pushbutton_apply->setEnabled(FALSE);
	qApp->restoreOverrideCursor();
}

int KernelConfigTab::readSysctl(QString strVarName)
{
	char tmpstr[500];
	QString strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+= "sysctl ";
	strShellCommand+= strVarName;
	strShellCommand+= " >"; strShellCommand+= TMP_DIR;
	strShellCommand+= "sysctl.tmp";
	if(system(strShellCommand))
		return -1;
	ifstream tmpfile(QString(TMP_DIR)+"sysctl.tmp");
	tmpfile.getline(tmpstr,500);
	if (strchr(tmpstr,' '))
		return atoi(strchr(tmpstr,' ')+1);
	return -1;
}

void KernelConfigTab::writeSysctl(QString strVarName,int nValue)
{
	QString strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+= "sysctl -w ";
	strShellCommand+= strVarName;
	strShellCommand+= "="; strShellCommand+= numtostr(nValue);
	strShellCommand+= " >/dev/null";
	system(strShellCommand);
}

void KernelConfigTab::getSysctlValues()
{
	checkbox_fw_enable->setChecked(readSysctl("net.inet.ip.fw.enable"));
	checkbox_fw_verbose->setChecked(readSysctl("net.inet.ip.fw.verbose"));
	checkbox_fw_debug->setChecked(readSysctl("net.inet.ip.fw.debug"));
	spinbox_fw_verbose_limit->setValue(readSysctl("net.inet.ip.fw.verbose_limit"));
	spinbox_fw_dyn_max->setValue(readSysctl("net.inet.ip.fw.dyn_max"));
	spinbox_fw_ack_lifetime->setValue(readSysctl("net.inet.ip.fw.dyn_ack_lifetime"));
	spinbox_fw_syn_lifetime->setValue(readSysctl("net.inet.ip.fw.dyn_syn_lifetime"));
	spinbox_fw_fin_lifetime->setValue(readSysctl("net.inet.ip.fw.dyn_fin_lifetime"));
	spinbox_fw_rst_lifetime->setValue(readSysctl("net.inet.ip.fw.dyn_rst_lifetime"));
	spinbox_fw_short_lifetime->setValue(readSysctl("net.inet.ip.fw.dyn_short_lifetime"));
	lineedit_fw_dyn_buckets->setText(numtostr(readSysctl("net.inet.ip.fw.dyn_buckets")));
	lineedit_fw_curr_dyn_buckets->setText(numtostr(readSysctl("net.inet.ip.fw.curr_dyn_buckets")));
	lineedit_fw_dyn_count->setText(numtostr(readSysctl("net.inet.ip.fw.dyn_count")));
	
}

void KernelConfigTab::setSysctlValues()
{
	writeSysctl("net.inet.ip.fw.enable",checkbox_fw_enable->isChecked() ? 1 : 0);
	writeSysctl("net.inet.ip.fw.verbose",checkbox_fw_verbose->isChecked() ? 1 : 0);
	writeSysctl("net.inet.ip.fw.debug",checkbox_fw_debug->isChecked() ? 1 : 0);
	writeSysctl("net.inet.ip.fw.verbose_limit",spinbox_fw_verbose_limit->value());
	writeSysctl("net.inet.ip.fw.dyn_max",spinbox_fw_dyn_max->value());
	writeSysctl("net.inet.ip.fw.dyn_ack_lifetime",spinbox_fw_ack_lifetime->value());
	writeSysctl("net.inet.ip.fw.dyn_syn_lifetime",spinbox_fw_syn_lifetime->value());
	writeSysctl("net.inet.ip.fw.dyn_fin_lifetime",spinbox_fw_fin_lifetime->value());
	writeSysctl("net.inet.ip.fw.dyn_rst_lifetime",spinbox_fw_rst_lifetime->value());
	writeSysctl("net.inet.ip.fw.dyn_short_lifetime",spinbox_fw_short_lifetime->value());
}

void KernelConfigTab::needUpdate()
{
	pushbutton_revert->setEnabled(TRUE);
	pushbutton_apply->setEnabled(TRUE);
}


