#include "comtypes.H"
#include "mainform.H"
#include "sudodialog.H"
#include "apprc.H"
//#include "allglobals.H"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <pwd.h>

#include <qwindowsstyle.h>
#include <qmotifstyle.h>
#include <qmotifplusstyle.h>
#include <qcdestyle.h>
#include <qplatinumstyle.h>
#include <qsgistyle.h>
#include <qinterlacestyle.h>

#include <qstring.h>
#include <qapplication.h>
#include <qlineedit.h>
#include <qmessagebox.h>

void load_protocol_list();
void load_service_list();
void load_interface_list();
void load_rc_settings();
void save_rc_settings();

ICMPTYPE_STRUCT ICMPTYPE_LIST[] =
{{0,"echo reply (0)"},{3,"destination unreachable (3)"},{4,"source quench (4)"},
{5, "redirect (5)"},{8,"echo request (8)"},
{9,"router advertisement (9)"},{10,"router solicitation (10)"},{11,"time-to-live exceeded (11)"},
{12,"IP header bad (12)"},{13,"timestamp request (13)"},
{14,"timestamp reply (14)"},{15,"information request (15)"},
{16,"information reply (16)"},{17,"address mask request (17)"},
{18,"address mask reply (18)"}};

#ifndef STRUCT_ICMPCODE
#define STRUCT_ICMPCODE
struct _icmpcode {
	int	code;
	char	*str;
};
#endif

APP_RC app_rc;

struct RC_ENTRY
{
	QString strName;
	QString * pstrValue;
};

RC_ENTRY rc_entry[] = {{"style",&(app_rc.style)},
				   {"browser",&(app_rc.browser)},
				   {"",NULL}
				};

QString * PROTOCOL_LIST;
_icmpcode * SERVICE_LIST;
QString * INTERFACE_LIST;
QString TCPFLAG_LIST[]={"fin","syn","rst","psh","ack","urg"};
QString IPOPTION_LIST[]={"ssrr","lsrr","rr","ts"};
QString TCPOPTION_LIST[]={"mss","window","sack","ts","cc"};

int PROTOCOL_LIST_SIZE;
int SERVICE_LIST_SIZE;
int INTERFACE_LIST_SIZE;
int TCPFLAG_LIST_SIZE = 6;
int IPOPTION_LIST_SIZE = 4;
int TCPOPTION_LIST_SIZE = 5;
int ICMPTYPE_LIST_SIZE = 15;

bool bUseSudo;
QString sudopassword;

int main(int argc,char * argv[])
{
	load_protocol_list();
	load_service_list();
	QApplication app(argc,argv);
	load_rc_settings();

	bUseSudo = false;

	if (getuid())
	{
		SudoDialog sudodialog(0,0,TRUE);
		if (sudodialog.exec())
		{
			bUseSudo = true;
			sudopassword = sudodialog.lineedit_password->text();
			QString strCommandline = "echo ";
			strCommandline+= sudopassword;
			strCommandline+= "|sudo -S ";
			strCommandline+= IPFW_COMMAND;
			strCommandline+= " list";
			if (system(strCommandline))
			{
				QMessageBox::critical(0,"Error","Incorrect password!");
				bUseSudo = false;
			}
		}

	}
	
	MainForm * pMainForm = new MainForm;
	app.setMainWidget(pMainForm);
	pMainForm->show();
	int nRet = app.exec();
	save_rc_settings();
	return nRet;
}

void load_protocol_list()
{
	protoent * pprotoent;
	int count = 0;
	QString tmpstr;
	while((pprotoent = getprotoent()) != NULL)
	{
		count++;
	}
	endprotoent();
	PROTOCOL_LIST_SIZE = count;
	PROTOCOL_LIST = new QString[PROTOCOL_LIST_SIZE];
	count = 0;
	while((pprotoent = getprotoent()) != NULL)
	{
		PROTOCOL_LIST[count] = pprotoent->p_name;
		count++;
	}
	endprotoent();
	int nfirst = 0;
	for (count = 0;count<PROTOCOL_LIST_SIZE;count++)
	{
		if ((PROTOCOL_LIST[count] == "ip")||
		    (PROTOCOL_LIST[count] == "tcp")||
		    (PROTOCOL_LIST[count] == "icmp")||
		    (PROTOCOL_LIST[count] == "udp"))
		{
			tmpstr = PROTOCOL_LIST[count];
			PROTOCOL_LIST[count] = PROTOCOL_LIST[nfirst];
			PROTOCOL_LIST[nfirst] = tmpstr;
			nfirst++;
		}
	}
}

void load_service_list()
{
	servent * pservent;
	int count = 0;
	while((pservent = getservent()) != NULL)
	{
		if (count == 2000) break;
		if (!strcmp(pservent->s_proto,"tcp"))
			count++;
	}
	endservent();
	SERVICE_LIST_SIZE = count;
	SERVICE_LIST = new _icmpcode[SERVICE_LIST_SIZE];
	count = 0;
	while((pservent = getservent()) != NULL)
	{
		if (count == 2000) break;
		if (!strcmp(pservent->s_proto,"tcp"))
		{
			SERVICE_LIST[count].str = new char[strlen(pservent->s_name)+1];
			strcpy(SERVICE_LIST[count].str, pservent->s_name);
			SERVICE_LIST[count].code = ntohs((short)pservent->s_port);
			count++;
		}
	}
	endservent();
}

void load_interface_list()
{
}

void load_rc_settings()
{
	char tmpstr[500];
	QString rcfile_name = getpwuid(getuid())->pw_dir;
	rcfile_name+="/.qtfwrc";
	ifstream rcfile(rcfile_name,ios::nocreate);
	app_rc.style = "Windows";
	app_rc.browser = "netscape";
	
	if (!rcfile.good())
		return;
	//  ***** Read [UI] section *****
	rcfile.getline(tmpstr,500);
	while (!rcfile.eof() && strncmp(tmpstr,"[UI]",4))
		rcfile.getline(tmpstr,500);
	rcfile.getline(tmpstr,500);
	while (!rcfile.eof() && strncmp(tmpstr,"[",1))
	{
		int i = 0;
		while (rc_entry[i].pstrValue)
		{
			if (!strncmp(tmpstr,rc_entry[i].strName,rc_entry[i].strName.length()))
			{
				if (strlen(strchr(tmpstr,'='))>1)
				{
					*(rc_entry[i].pstrValue) = strchr(tmpstr,'=') + 1;
				}
			}
			i++;
		}
		rcfile.getline(tmpstr,500);
	}
	rcfile.seekg(0,ios::beg);
	// ***** Read saved hostnames *****
	rcfile.getline(tmpstr,500);
	while (!rcfile.eof() && strncmp(tmpstr,"[HOSTNAMES]",4))
		rcfile.getline(tmpstr,500);
	rcfile.getline(tmpstr,500);
	while (!rcfile.eof() && strncmp(tmpstr,"[",1))
	{
		app_rc.hostnames.append(QString(tmpstr));
		rcfile.getline(tmpstr,500);
	}
}

void save_rc_settings()
{
	QString rcfile_name = getpwuid(getuid())->pw_dir;
	rcfile_name+="/.qtfwrc";
	ofstream rcfile(rcfile_name,ios::trunc|ios::out);
	if (!rcfile.good())
		return;
	rcfile<<"[UI]" <<endl;
	int i = 0;
	while (rc_entry[i++].strName != "")
		rcfile<<rc_entry[i-1].strName <<"=" <<*(rc_entry[i-1].pstrValue) <<endl;
	rcfile <<"[HOSTNAMES]" <<endl;
	QStringList::Iterator it;
	for (it = app_rc.hostnames.begin();it != app_rc.hostnames.end();it++)
		rcfile <<*it <<endl;
	rcfile <<"[END]";
}



