#include "comtypes.H"
#include "mainform.H"
#include "kernelconfigtab.H"
#include "allglobals.H"
#include "ruleeditdialog.H"
#include "aboutdlg.H"
#include "pixmaps.H"
#include "scriptdialog.H"

#include <fstream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>

#include <qapplication.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qtoolbutton.h>
#include <qlistbox.h>
#include <qstringlist.h>
#include <qfiledialog.h>
#include <qpopupmenu.h>
#include <qtabwidget.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qframe.h>

QString numtostr(int num);
/* 
 *  Constructs a MainForm which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
MainForm::MainForm( QWidget* parent,  const char* name, WFlags fl )
    : MainFormBase( parent, name, fl )
{
	tab_kernel = NULL;

	if (!getuid() || bUseSudo)
	{
		tab_kernel = new KernelConfigTab(tabwidget,"tab_kernel",0);
		tabwidget->insertTab(tab_kernel,tr("Kernel configuration"));
	}

	qlistview_rules = new QListView(tab);

	gridlayout_tab1 = new QGridLayout(tab,1,1,2,5);
	gridlayout_tab_repository = new QGridLayout(tab_repository,1,0,2,5);
	gridlayout_repository_buttons = new QGridLayout((QWidget*)NULL,0,4,2,5);
	gridlayout_tab_repository->addLayout(gridlayout_repository_buttons,1,0);
	
	gridlayout_tab_repository->addWidget(listbox_repository,0,0);
	gridlayout_repository_buttons->addWidget(pushbutton_repository_load,0,2);
	gridlayout_repository_buttons->addWidget(pushbutton_repository_save,0,3);
	gridlayout_repository_buttons->addWidget(pushbutton_repository_delete,0,4);
	gridlayout_repository_buttons->addWidget(pushbutton_repository_clear,0,5);
	gridlayout_repository_buttons->addWidget(pushbutton_repository_new,0,0);
	gridlayout_repository_buttons->addWidget(pushbutton_repository_edit,0,1);
	gridlayout_buttons = new QGridLayout((QWidget*)NULL,0,4,2,5);
	gridlayout_tab1->addLayout(gridlayout_buttons,1,0);
	gridlayout_tab1->addWidget(qlistview_rules,0,0);
	gridlayout_buttons->addWidget(pushbutton_refresh,0,0);
	gridlayout_buttons->addWidget(pushbutton_delete,0,3);
	gridlayout_buttons->addWidget(pushbutton_flush,0,4);
	gridlayout_buttons->addWidget(pushbutton_edit_rule,0,2);
	gridlayout_buttons->addWidget(pushbutton_add_rule,0,1);
	gridlayout_movebuttons = new QGridLayout((QWidget*)NULL,5,0,2,5);
	gridlayout_tab1->addLayout(gridlayout_movebuttons,0,1);
	gridlayout_movebuttons->addWidget(toolbutton_up,2,0);
	gridlayout_movebuttons->addWidget(toolbutton_down,3,0);
	qlistview_rules->addColumn("Number");
	qlistview_rules->addColumn(" ");
	qlistview_rules->addColumn("Rule");

	pixmap_allow = new QPixmap(( const char** )image_allow_data );
	pixmap_deny = new QPixmap(( const char** )image_deny_data );
	pixmap_fwd = new QPixmap(( const char** )image_fwd_data );
	pixmap_count = new QPixmap(( const char** )image_count_data );
	
	popupmenu_rules = new QPopupMenu;
	popupmenu_rules->insertItem("Refresh",this,SLOT(refreshRules()));
	popupmenu_rules->insertItem("New rule...",this,SLOT(newRule()));
	popupmenu_rules->insertItem("Edit rule...",this,SLOT(editRule()));
	popupmenu_rules->insertItem("Delete",this,SLOT(deleteRule()));
	popupmenu_rules->insertItem("Flush",this,SLOT(flushIpfw()));
	popupmenu_rules->insertSeparator();
	popupmenu_rules->insertItem("Add to repository",this,SLOT(addToRepository()));
	popupmenu_rules->insertItem("Add ALL to repository",this,SLOT(addAllToRepository()));
	
	popupmenu_repository = new QPopupMenu;
	if (!getuid() || bUseSudo)
	{
		popupmenu_repository->insertItem("Add to ipfw list",this,SLOT(activateRepositoryRule()));
		popupmenu_repository->insertItem("Add ALL to ipfw list",this,SLOT(activateAllRepositoryRules()));
	}
	popupmenu_repository->insertSeparator();
	popupmenu_repository->insertItem("New...",this,SLOT(newRepositoryRule()));
	popupmenu_repository->insertItem("Edit...",this,SLOT(editRepositoryRule()));
	popupmenu_repository->insertItem("Load...",this,SLOT(loadRepository()));
	popupmenu_repository->insertItem("Save...",this,SLOT(saveRepository()));
	popupmenu_repository->insertItem("Delete",this,SLOT(deleteRepositoryRule()));
	popupmenu_repository->insertItem("Clear",this,SLOT(clearRepository()));
	
	scriptdialog = new ScriptDialog();
	scriptdialog->mainform = this;
	scriptdialog->kerneltab = tab_kernel;
	// *** If not root and not using sudo, disable tab1 and some other elements.

	if (getuid() && !bUseSudo)
	{
		
		tab->setEnabled(FALSE);
		
	}

	connect(pushbutton_refresh,SIGNAL(clicked()),this,SLOT(refreshRules()));
	connect(pushbutton_delete,SIGNAL(clicked()),this,SLOT(deleteRule()));
	connect(pushbutton_flush,SIGNAL(clicked()),this,SLOT(flushIpfw()));
	connect(pushbutton_edit_rule,SIGNAL(clicked()),this,SLOT(editRule()));
	connect(pushbutton_add_rule,SIGNAL(clicked()),this,SLOT(newRule()));
	connect(toolbutton_quit,SIGNAL(clicked()),this,SLOT(onQuit()));
	connect(toolbutton_about,SIGNAL(clicked()),this,SLOT(about()));
	connect(toolbutton_help,SIGNAL(clicked()),this,SLOT(help()));
	connect(pushbutton_repository_load,SIGNAL(clicked()),this,SLOT(loadRepository()));
	connect(pushbutton_repository_save,SIGNAL(clicked()),this,SLOT(saveRepository()));
	connect(pushbutton_repository_delete,SIGNAL(clicked()),this,SLOT(deleteRepositoryRule()));
	connect(pushbutton_repository_clear,SIGNAL(clicked()),this,SLOT(clearRepository()));
	connect(pushbutton_repository_new,SIGNAL(clicked()),this,SLOT(newRepositoryRule()));
	connect(pushbutton_repository_edit,SIGNAL(clicked()),this,SLOT(editRepositoryRule()));
	connect(qlistview_rules,SIGNAL(rightButtonPressed(QListViewItem*,const QPoint&,int)),this,SLOT(onPopupRules(QListViewItem *,const QPoint&,int)));
	connect(listbox_repository,SIGNAL(rightButtonPressed(QListBoxItem*,const QPoint&)),this,SLOT(onPopupRepository(QListBoxItem *,const QPoint&)));
	connect(toolbutton_up,SIGNAL(clicked()),this,SLOT(moveUp()));
	connect(toolbutton_down,SIGNAL(clicked()),this,SLOT(moveDown()));
	connect(toolbutton_script,SIGNAL(clicked()),this,SLOT(createScript()));
	if (!getuid() || bUseSudo)
		refreshRules();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
MainForm::~MainForm()
{
    // no need to delete child widgets, Qt does it all for us
}

void MainForm::refreshRules()
{
	qApp->setOverrideCursor(waitCursor);
	QListViewItem * tmpItem;
	FWRule tmpRule;
	char tmpstr[500];
	int nlastitem = 0;
	if (qlistview_rules->currentItem())
	{
		nlastitem = atoi(qlistview_rules->currentItem()->text(0));
	}
	qlistview_rules->clear();
	QString strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" list >";
	strShellCommand+=TMP_DIR;
	strShellCommand+="ipfwlist.tmp";

	if(system(strShellCommand))
	{
		qApp->restoreOverrideCursor();
		return;
	}
	
	ifstream tmpfile(QString(TMP_DIR)+"ipfwlist.tmp");
	while (!tmpfile.eof())
	{
		tmpfile.getline(tmpstr,500);
		if (strlen(tmpstr) > 1)
		{
			tmpRule.readString(tmpstr);
			tmpItem = new QListViewItem(qlistview_rules,numtostr(atoi(tmpstr)).rightJustify(5,'0'),"",tmpstr);
			if ((tmpRule.ipfw_action == "allow")||(tmpRule.ipfw_action == "pipe")||(tmpRule.ipfw_action == "queue")||(tmpRule.ipfw_action == "skipto")||(tmpRule.ipfw_action == "check-state"))
				tmpItem->setPixmap(1,*pixmap_allow);
			else
			if ((tmpRule.ipfw_action == "deny")||(tmpRule.ipfw_action == "unreach")||(tmpRule.ipfw_action == "reject")||(tmpRule.ipfw_action == "reset"))
				tmpItem->setPixmap(1,*pixmap_deny);
			else
			if ((tmpRule.ipfw_action == "divert")||(tmpRule.ipfw_action == "tee")||(tmpRule.ipfw_action == "fwd")||(tmpRule.ipfw_action == "forward"))
				tmpItem->setPixmap(1,*pixmap_fwd);
			else
			if ((tmpRule.ipfw_action == "count"))
				tmpItem->setPixmap(1,*pixmap_count);
			tmpRule.clear();
		}
	}
	tmpItem = qlistview_rules->firstChild();
	while (tmpItem)
	{
		if (atoi(tmpItem->text(0)) >= nlastitem)
		{
			qlistview_rules->setCurrentItem(tmpItem);
			qlistview_rules->setSelected(tmpItem,TRUE);
			break;
		}
		tmpItem = tmpItem->nextSibling();
	}
	qApp->restoreOverrideCursor();
}

void MainForm::deleteRule()
{
	if (!qlistview_rules->currentItem())
		return;

	QString strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q delete ";
	strShellCommand+=qlistview_rules->currentItem()->text(0);

	if(system(strShellCommand))
		return;
	refreshRules();
}

void MainForm::flushIpfw()
{
	QString strShellCommand = "";
	if (QMessageBox::warning(this,"Flushing ipfw","You are going to delete \
all rules from the list.\nAre You sure?","Yes","No"))
		return;
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q flush";

	if(system(strShellCommand))
		return;
	refreshRules();
}

void MainForm::editRule()
{
	if (!qlistview_rules->currentItem())
		return;
	QString strShellCommand = "";
	QString strItemToDelete = qlistview_rules->currentItem()->text(0);
	QString strRuleToDelete = qlistview_rules->currentItem()->text(2);
	MyRuleEditDialog edit_dlg(qlistview_rules->currentItem()->text(2),this,0,TRUE);
	if (edit_dlg.exec())
	{
		while (ruleExists(edit_dlg.fwrule.ipfw_number)&&(edit_dlg.fwrule.ipfw_number!=atoi(qlistview_rules->currentItem()->text(0))))
			edit_dlg.fwrule.ipfw_number++;
		edit_dlg.strRule = edit_dlg.fwrule.formRule();
		if (bUseSudo)
			strShellCommand = "echo " + sudopassword + "|sudo -S ";
		strShellCommand+=IPFW_COMMAND;
		strShellCommand+=" -q delete " + strItemToDelete;
		if(system(strShellCommand))
			return;
		
		strShellCommand = "";
		if (bUseSudo)
			strShellCommand = "echo " + sudopassword + "|sudo -S ";
		strShellCommand+=IPFW_COMMAND;
		strShellCommand+=" -q " + edit_dlg.strRule;
		strShellCommand+=" 2>";
		strShellCommand+=TMP_DIR;
		strShellCommand+="ipfwlist.tmp";
		if(system(strShellCommand))
		{
			QString fname = QString(TMP_DIR)+"ipfwlist.tmp";
			ifstream ifstr(fname);
			char error_msg[500];
			ifstr.getline(error_msg,499);
			QMessageBox::critical(this,"ipfw error",error_msg);
			strShellCommand ="";
			if (bUseSudo)
				strShellCommand = "echo " + sudopassword + "|sudo -S ";
			strShellCommand+=IPFW_COMMAND;
			strShellCommand+=" -q add " + strRuleToDelete;
			strShellCommand+=" 2>";
			strShellCommand+=TMP_DIR;
			strShellCommand+="ipfwlist.tmp";
			system(strShellCommand);
			refreshRules();
			if (!QMessageBox::warning(this,"Move this rule to repository","The rule You've edited contains errors.\nDo You want to save this rule in repository\nand edit it later?","Yes","No"))
			{
				listbox_repository->insertItem(edit_dlg.strRule.right(edit_dlg.strRule.length()-edit_dlg.strRule.find(' ')-1));
				listbox_repository->sort();
				tabwidget->setCurrentPage(1);
			}
			return;
		}
		refreshRules();
	}
}

void MainForm::newRule()
{
	QString strBasicRule = "100";
	if (qlistview_rules->currentItem())
		strBasicRule = numtostr((atoi(qlistview_rules->currentItem()->text(0))==65535) ? 100 : (atoi(qlistview_rules->currentItem()->text(0))+1));
	strBasicRule+=" allow ip from any to any";
	MyRuleEditDialog edit_dlg(strBasicRule,this,0,TRUE);
	QString strShellCommand="";
	if (edit_dlg.exec())
	{
		while (ruleExists(edit_dlg.fwrule.ipfw_number))
			edit_dlg.fwrule.ipfw_number++;
		edit_dlg.strRule = edit_dlg.fwrule.formRule();		
		if (bUseSudo)
			strShellCommand = "echo " + sudopassword + "|sudo -S ";
		strShellCommand+=IPFW_COMMAND;
		strShellCommand+=" -q " + edit_dlg.strRule;
		strShellCommand+=" 2>";
		strShellCommand+=TMP_DIR;
		strShellCommand+="ipfwlist.tmp";
		if(system(strShellCommand))
		{
			QString fname = QString(TMP_DIR)+"ipfwlist.tmp";
			ifstream ifstr(fname);
			char error_msg[500];
			ifstr.getline(error_msg,499);
			QMessageBox::critical(this,"ipfw error",error_msg);
			if (!QMessageBox::warning(this,"Move this rule to repository","The rule You've created contains errors.\nDo You want to save this rule in repository\nand edit it later?","Yes","No"))
			{
				listbox_repository->insertItem(edit_dlg.strRule.right(edit_dlg.strRule.length()-edit_dlg.strRule.find(' ')-1));
				listbox_repository->sort();
				tabwidget->setCurrentPage(1);
			}
			refreshRules();
			return;
		}
		refreshRules();
		QListViewItem * tmpItem = qlistview_rules->firstChild();
		while (tmpItem)
		{
			if (atoi(tmpItem->text(0)) == edit_dlg.fwrule.ipfw_number)
			{
				qlistview_rules->setCurrentItem(tmpItem);
				qlistview_rules->setSelected(tmpItem,TRUE);
				break;
			}
			tmpItem = tmpItem->nextSibling();
		}
	}	
}

void MainForm::about()
{
	AboutDialog about_dlg(this,0,TRUE);
	about_dlg.exec();
}

void MainForm::loadRepository()
{
	if (listbox_repository->count() != 0)
		if (!QMessageBox::warning(this,"Clear repository","Do You want to delete existing rules?","Yes","No"))
			listbox_repository->clear();
	QFileDialog fdlg(".",QString::null,this,"Load rules from file:",TRUE);
	fdlg.setMode(QFileDialog::ExistingFile);
	if (fdlg.exec())
	{
		if (fdlg.selectedFile().isNull())
			return;
		ifstream if_rules(fdlg.selectedFile());
		if (!if_rules.good())
		{
			QMessageBox::critical(this,"Error!","Error opening file!");
			return;
		}
		char tmpstr[500];
		FWRule tmprule;
		while (!if_rules.eof())
		{
			if_rules.getline(tmpstr,500);
			if (strlen(tmpstr) < 2)
				continue;
			tmprule.m_strError = "";
			tmprule.readString(tmpstr);
			if (tmprule.m_strError == "")
				listbox_repository->insertItem(tmpstr);	
			tmprule.clear();
		}
		listbox_repository->sort();
	}
	
}

void MainForm::saveRepository()
{
	if (listbox_repository->count() == 0)
		return;
	QFileDialog fdlg(".",QString::null,this,"Save rules to file:",TRUE);
	fdlg.setMode(QFileDialog::AnyFile);
	if (fdlg.exec())
	{
		if (fdlg.selectedFile().isNull())
			return;
		ofstream of_rules(fdlg.selectedFile(),ios::out|ios::trunc);
		if (!of_rules.good())
		{
			QMessageBox::critical(this,"Error!","Error opening file!");
			return;
		}
		for (unsigned int n = 0;n < listbox_repository->count();n++)
		{
			of_rules <<listbox_repository->text(n) <<endl;
		}
	}
}

void MainForm::deleteRepositoryRule()
{
	unsigned int n;
	QStringList selectedItemsList;
	for (n = 0; n < listbox_repository->count(); n++)
		if (listbox_repository->isSelected(n))
		{
			selectedItemsList.append(listbox_repository->text(n));
		}
	for (QStringList::Iterator it = selectedItemsList.begin();it != selectedItemsList.end();it++)
	{
		listbox_repository->removeItem(listbox_repository->index(listbox_repository->findItem(*it)));
	}
	listbox_repository->sort();
}

void MainForm::clearRepository()
{
	if (listbox_repository->count() == 0)
		return;
	if (QMessageBox::warning(this,"Clear repository","You are going to delete all rules.\nAre You sure?","Yes","No"))
		return;
	listbox_repository->clear();
}

void MainForm::addToRepository()
{
	if (!qlistview_rules->currentItem())
		return;
	listbox_repository->insertItem(qlistview_rules->currentItem()->text(2));
	listbox_repository->sort();
//	tabwidget->setCurrentPage(1);
}

void MainForm::addAllToRepository()
{
	QListViewItem * listitem = qlistview_rules->firstChild();
	if (listitem)
		listbox_repository->insertItem(listitem->text(2));
	else return;
	while ((listitem = listitem->nextSibling()) != NULL)
		listbox_repository->insertItem(listitem->text(2));
	listbox_repository->sort();
	tabwidget->setCurrentPage(1);
}

void MainForm::editRepositoryRule()
{
	QString strTruncatedRule;
	int nCurrentItem = listbox_repository->currentItem();
	if (nCurrentItem < 0) return;
	MyRuleEditDialog edit_dlg(listbox_repository->currentText(),this,0,TRUE);
	if (edit_dlg.exec())
	{
		strTruncatedRule = edit_dlg.strRule;
		strTruncatedRule.remove(0,strTruncatedRule.find(" ")+1);
		listbox_repository->changeItem(strTruncatedRule,nCurrentItem);
		listbox_repository->sort();
	}
	
}

void MainForm::newRepositoryRule()
{
	QString strTruncatedRule;
	MyRuleEditDialog edit_dlg("",this,0,TRUE);
	if (edit_dlg.exec())
	{
		strTruncatedRule = edit_dlg.strRule;
		strTruncatedRule.remove(0,strTruncatedRule.find(" ")+1);
		listbox_repository->insertItem(strTruncatedRule);
		listbox_repository->sort();
	}
}

void MainForm::activateRepositoryRule()
{
	if (listbox_repository->currentItem() < 0)
		return;
	QString strShellCommand="";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q add " + listbox_repository->currentText();
	strShellCommand+=" 2>";
	strShellCommand+=TMP_DIR;
	strShellCommand+="ipfwlist.tmp";
	if(system(strShellCommand))
	{
		QString fname = QString(TMP_DIR)+"ipfwlist.tmp";
		ifstream ifstr(fname);
		char error_msg[500];
		ifstr.getline(error_msg,499);
		QMessageBox::critical(this,"ipfw error",error_msg);
		refreshRules();
		return;
	}
	refreshRules();
//	tabwidget->setCurrentPage(0);
}

void MainForm::activateAllRepositoryRules()
{
	qApp->setOverrideCursor(waitCursor);
	QString strShellCommand;
	for (unsigned int i = 0; i<listbox_repository->count();i++)
	{
		strShellCommand="";
		if (bUseSudo)
			strShellCommand = "echo " + sudopassword + "|sudo -S ";
		strShellCommand+=IPFW_COMMAND;
		strShellCommand+=" -q add " + listbox_repository->text(i);
		strShellCommand+=" 2>";
		strShellCommand+=TMP_DIR;
		strShellCommand+="ipfwlist.tmp";
		if(system(strShellCommand))
		{
			QString fname = QString(TMP_DIR)+"ipfwlist.tmp";
			ifstream ifstr(fname);
			char error_msg[500];
			ifstr.getline(error_msg,499);
			QMessageBox::critical(this,"ipfw error",error_msg);
			refreshRules();
		}
	}
	qApp->restoreOverrideCursor();
	refreshRules();
	tabwidget->setCurrentPage(0);
}
void MainForm::onPopupRules(QListViewItem * pItem,const QPoint& point,int column)
{
	popupmenu_rules->exec(point);
}

void MainForm::onPopupRepository(QListBoxItem * pItem,const QPoint& point)
{
	popupmenu_repository->exec(point);
}

void MainForm::moveUp()
{
	if (!qlistview_rules->currentItem())
		return;
	if (!qlistview_rules->currentItem()->itemAbove())
		return;
	if (qlistview_rules->currentItem()->text(0) == "65535")
		return;
	QString strRuleToMove = qlistview_rules->currentItem()->text(2);
	QString strNumRuleToMove = qlistview_rules->currentItem()->text(0);
	QString strRuleAbove = qlistview_rules->currentItem()->itemAbove()->text(2);
	QString strNumRuleAbove = qlistview_rules->currentItem()->itemAbove()->text(0);
	FWRule rule_to_move,rule_above;
	char *str1,*str2;
	str1 = new char[strRuleToMove.length()+1];
	str2 = new char[strRuleAbove.length()+1];
	strcpy(str1,strRuleToMove);
	strcpy(str2,strRuleAbove);
	rule_to_move.readString(str1);
	rule_above.readString(str2);
	unsigned short int nTmp = rule_above.ipfw_number;
	rule_above.ipfw_number = rule_to_move.ipfw_number;
	rule_to_move.ipfw_number = nTmp;
	
	QString strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q delete ";
	strShellCommand+=strNumRuleToMove;
	if(system(strShellCommand))
		return;
	strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q delete ";
	strShellCommand+=strNumRuleAbove;
	if(system(strShellCommand))
		return;
	strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q " + rule_to_move.formRule();
	strShellCommand+=" 2>";
	strShellCommand+=TMP_DIR;
	strShellCommand+="ipfwlist.tmp";
	system(strShellCommand);
	strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q " + rule_above.formRule();
	strShellCommand+=" 2>";
	strShellCommand+=TMP_DIR;
	strShellCommand+="ipfwlist.tmp";
	system(strShellCommand);
	refreshRules();
	QListViewItem * tmpItem = qlistview_rules->firstChild();
	while (tmpItem)
	{
		if (atoi(tmpItem->text(0)) == rule_to_move.ipfw_number)
		{
			qlistview_rules->setCurrentItem(tmpItem);
			qlistview_rules->setSelected(tmpItem,TRUE);
			break;
		}
		tmpItem = tmpItem->nextSibling();
	}	
	delete []str1;
	delete []str2;
	rule_to_move.clear();
	rule_above.clear();
}

void MainForm::moveDown()
{
	if (!qlistview_rules->currentItem())
		return;
	if (!qlistview_rules->currentItem()->itemBelow())
		return;
	if (qlistview_rules->currentItem()->text(0) == "65535")
		return;
	if (qlistview_rules->currentItem()->itemBelow()->text(0) == "65535")
		return;
	QString strRuleToMove = qlistview_rules->currentItem()->text(2);
	QString strNumRuleToMove = qlistview_rules->currentItem()->text(0);
	QString strRuleBelow = qlistview_rules->currentItem()->itemBelow()->text(2);
	QString strNumRuleBelow = qlistview_rules->currentItem()->itemBelow()->text(0);
	FWRule rule_to_move,rule_below;
	char *str1,*str2;
	str1 = new char[strRuleToMove.length()+1];
	str2 = new char[strRuleBelow.length()+1];
	strcpy(str1,strRuleToMove);
	strcpy(str2,strRuleBelow);
	rule_to_move.readString(str1);
	rule_below.readString(str2);
	unsigned short int nTmp = rule_below.ipfw_number;
	rule_below.ipfw_number = rule_to_move.ipfw_number;
	rule_to_move.ipfw_number = nTmp;
	
	QString strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q delete ";
	strShellCommand+=strNumRuleToMove;
	if(system(strShellCommand))
		return;
	strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q delete ";
	strShellCommand+=strNumRuleBelow;
	if(system(strShellCommand))
		return;
	strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q " + rule_to_move.formRule();
	strShellCommand+=" 2>";
	strShellCommand+=TMP_DIR;
	strShellCommand+="ipfwlist.tmp";
	system(strShellCommand);
	strShellCommand = "";
	if (bUseSudo)
		strShellCommand = "echo " + sudopassword + "|sudo -S ";
	strShellCommand+=IPFW_COMMAND;
	strShellCommand+=" -q " + rule_below.formRule();
	strShellCommand+=" 2>";
	strShellCommand+=TMP_DIR;
	strShellCommand+="ipfwlist.tmp";
	system(strShellCommand);
	refreshRules();
	QListViewItem * tmpItem = qlistview_rules->firstChild();
	while (tmpItem)
	{
		if (atoi(tmpItem->text(0)) == rule_to_move.ipfw_number)
		{
			qlistview_rules->setCurrentItem(tmpItem);
			qlistview_rules->setSelected(tmpItem,TRUE);
			break;
		}
		tmpItem = tmpItem->nextSibling();
	}	
	delete []str1;
	delete []str2;
	rule_to_move.clear();
	rule_below.clear();
	return;
}

bool MainForm::ruleExists(int number)
{
	QListViewItem * tmpItem = qlistview_rules->firstChild();
	while (tmpItem)
	{
		if (atoi(tmpItem->text(0)) == number)
			return true;
		tmpItem = tmpItem->nextSibling();
	}
	return false;
}

void MainForm::onQuit()
{
	if (QMessageBox::warning(this,"Quitting...","Are You sure?","Yes","No"))
		return;
	qApp->quit();
}

void MainForm::closeEvent(QCloseEvent * e)
{
	onQuit();
}

void MainForm::help()
{

}

void MainForm::createScript()
{
	scriptdialog->show();
}
