#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qstringlist.h>
#include <qbuttongroup.h>

#ifndef STRUCT_ICMPCODE
#define STRUCT_ICMPCODE
struct _icmpcode {
	int	code;
	char	*str;
};
#endif

#include "comtypes.H"
#include "allglobals.H"
#include "ruleeditdialog.H"
#include "optionlistdialog.H"

#define         ICMP_UNREACH_NET        0               /* bad net */
#define         ICMP_UNREACH_HOST       1               /* bad host */
#define         ICMP_UNREACH_PROTOCOL   2               /* bad protocol */
#define         ICMP_UNREACH_PORT       3               /* bad port */
#define         ICMP_UNREACH_NEEDFRAG   4               /* IP_DF caused drop */
#define         ICMP_UNREACH_SRCFAIL    5               /* src route failed */
#define         ICMP_UNREACH_NET_UNKNOWN 6              /* unknown net */
#define         ICMP_UNREACH_HOST_UNKNOWN 7             /* unknown host */
#define         ICMP_UNREACH_ISOLATED   8               /* src host isolated */
#define         ICMP_UNREACH_NET_PROHIB 9               /* prohibited access */
#define         ICMP_UNREACH_HOST_PROHIB 10             /* ditto */
#define         ICMP_UNREACH_TOSNET     11              /* bad tos for net */
#define         ICMP_UNREACH_TOSHOST    12              /* bad tos for host */
#define         ICMP_UNREACH_FILTER_PROHIB 13           /* admin prohib */
#define         ICMP_UNREACH_HOST_PRECEDENCE 14         /* host prec vio. */
#define         ICMP_UNREACH_PRECEDENCE_CUTOFF 15       /* prec cutoff */



static struct _icmpcode _icmpcodes[] = {
      { ICMP_UNREACH_NET,		"net" },
      { ICMP_UNREACH_HOST,		"host" },
      { ICMP_UNREACH_PROTOCOL,		"protocol" },
      { ICMP_UNREACH_PORT,		"port" },
      { ICMP_UNREACH_NEEDFRAG,		"needfrag" },
      { ICMP_UNREACH_SRCFAIL,		"srcfail" },
      { ICMP_UNREACH_NET_UNKNOWN,	"net-unknown" },
      { ICMP_UNREACH_HOST_UNKNOWN,	"host-unknown" },
      { ICMP_UNREACH_ISOLATED,		"isolated" },
      { ICMP_UNREACH_NET_PROHIB,	"net-prohib" },
      { ICMP_UNREACH_HOST_PROHIB,	"host-prohib" },
      { ICMP_UNREACH_TOSNET,		"tosnet" },
      { ICMP_UNREACH_TOSHOST,		"toshost" },
      { ICMP_UNREACH_FILTER_PROHIB,	"filter-prohib" },
      { ICMP_UNREACH_HOST_PRECEDENCE,	"host-precedence" },
      { ICMP_UNREACH_PRECEDENCE_CUTOFF,	"precedence-cutoff" },
      { 0, NULL }
};
QString numtostr(int num);


void print_if_list(QStringList & stringlist);

/* 
 *  Constructs a MyRuleEditDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
MyRuleEditDialog::MyRuleEditDialog(QString qstrRule, QWidget* parent,  const char* name, bool modal, WFlags fl )
    : RuleEditDialog( parent, name, modal, fl )
{
/*	pushbutton_ok->setGeometry( QRect( 5, 475, 70, 30 ) );
	pushbutton_cancel->setGeometry( QRect( 90, 475, 70, 30 ) );*/
	bNoUpdateNow = true;
	strRule = qstrRule;
	if (strRule == "")
		strRule = "100 allow ip from any to any";
	
	// Load lists and set default values here
	int i;
	passwd * ppass;
	group * pgroup;

	while ((ppass=getpwent())!= NULL)
		combobox_uid->insertItem(ppass->pw_name);
	endpwent();
	combobox_uid->setCurrentItem(0);
	while ((pgroup=getgrent())!= NULL)
		combobox_gid->insertItem(pgroup->gr_name);
	endgrent();	
	combobox_gid->setCurrentItem(0);
	
	for (i=0;i<90;i++)
	{
		combobox_teeport->insertItem(SERVICE_LIST[i].str);
	}
	combobox_teeport->setCurrentItem(0);
	
	for (i=0;i<PROTOCOL_LIST_SIZE;i++)
	{
		combobox_proto->insertItem(PROTOCOL_LIST[i]);
	}
	combobox_proto->setCurrentItem(0);
	char tmpstr[100];
	for (i = 0;i<16;i++)
	{
		sprintf(tmpstr,"%d - %s",i,_icmpcodes[i].str);
		combobox_unreach_code->insertItem(tmpstr);
	}
	combobox_unreach_code->setCurrentItem(0);
	lineedit_fwd_address->setText("127.0.0.1");
	// Now retrieve available network interfaces

	QStringList iflist;
	print_if_list(iflist);
	QStringList::Iterator itList;
	for (itList = iflist.begin();itList != iflist.end();++itList)
	{
		combobox_via->insertItem(*itList);
	}
	combobox_via->setCurrentItem(0);

	// ***** Fill source and destination address comboboxes
	QStringList::Iterator itHname;
	for (itHname = app_rc.hostnames.begin();itHname != app_rc.hostnames.end();itHname++)
	{
		combobox_src_address->insertItem(*itHname);
		combobox_dst_address->insertItem(*itHname);
	}
	
	//***************************************
	lineedit_icmptypes->setReadOnly(true);
	lineedit_tcpflags->setReadOnly(true);
	lineedit_tcpoptions->setReadOnly(true);
	lineedit_ipoptions->setReadOnly(true);
	lineedit_src_portlist->setReadOnly(true);
	lineedit_dst_portlist->setReadOnly(true);
	setElements();
	updateElements();
	updateRule();
	connect(pushbutton_ok,SIGNAL(clicked()),this,SLOT(onOK()));
	connect(pushbutton_cancel,SIGNAL(clicked()),this,SLOT(reject()));
	bNoUpdateNow = false;
}


/*  
 *  Destroys the object and frees any allocated resources
 */
MyRuleEditDialog::~MyRuleEditDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

void MyRuleEditDialog::setElements()
{
	int i;
	char * tmpstr = new char[strRule.length()+1];
	strcpy(tmpstr,strRule);
	fwrule.readString(tmpstr);
//	fwrule.dump(cout);
	spinbox_rule_number->setValue(fwrule.ipfw_number);
	if (fwrule.ipfw_bprob)
	{
		checkbox_probability->setChecked(TRUE);
		spinbox_probability->setValue((int)(fwrule.ipfw_matchprob*100));
	}
	if (fwrule.ipfw_blog)
	{
		checkbox_log->setChecked(TRUE);
		if (fwrule.ipfw_blogamount)
		{
			checkbox_logamount->setChecked(TRUE);
			spinbox_logamount->setValue((int)fwrule.ipfw_logamount);
		}
	}
	for (i = 0;i<combobox_proto->count();i++)
		if (combobox_proto->text(i) == fwrule.ipfw_protocol)
		{
			combobox_proto->setCurrentItem(i);
			break;
		}
	if (i == combobox_proto->count())
	{
		combobox_proto->insertItem(fwrule.ipfw_protocol,0);
		combobox_proto->setCurrentItem(0);
	}
	if (fwrule.ipfw_direction == "in")
		radiobutton_incoming->setChecked(TRUE);
	else
	if (fwrule.ipfw_direction == "out")
		radiobutton_outcoming->setChecked(TRUE);
	else
		radiobutton_nodirection->setChecked(TRUE);
	if (fwrule.ipfw_via != "")
	{
		checkbox_via->setChecked(TRUE);
		for (i = 0;i<combobox_via->count();i++)
			if (combobox_via->text(i) == fwrule.ipfw_via)
			{
				combobox_via->setCurrentItem(i);
				break;
			}
		if (i == combobox_via->count())
		{
			combobox_via->insertItem(fwrule.ipfw_via,0);
			combobox_via->setCurrentItem(0);
		}
	}
	
	if (fwrule.ipfw_option_keepstate)
		checkbox_keepstate->setChecked(TRUE);
	if (fwrule.ipfw_option_fragment)
		checkbox_fragment->setChecked(TRUE);
	if (fwrule.ipfw_option_setup)
		checkbox_setup->setChecked(TRUE);
	if (fwrule.ipfw_option_established)
		checkbox_established->setChecked(TRUE);
	if (fwrule.ipfw_option_bridged)
		checkbox_bridged->setChecked(TRUE);	
	if (fwrule.ipfw_uid != "")
	{
		for (i = 0;i<combobox_uid->count();i++)
			if (combobox_uid->text(i) == fwrule.ipfw_uid)
			{
				combobox_uid->setCurrentItem(i);
				break;
			}
		if (i == combobox_uid->count())
		{
			combobox_uid->insertItem(fwrule.ipfw_uid,0);
			combobox_uid->setCurrentItem(0);
		}
		checkbox_uid->setChecked(TRUE);
	}
	if (fwrule.ipfw_gid != "")
	{
		checkbox_gid->setChecked(TRUE);
		for (i = 0;i<combobox_gid->count();i++)
			if (combobox_gid->text(i) == fwrule.ipfw_gid)
			{
				combobox_gid->setCurrentItem(i);
				break;
			}
		if (i == combobox_gid->count())
		{
			combobox_gid->insertItem(fwrule.ipfw_gid,0);
			combobox_gid->setCurrentItem(0);
		}
	}
	if (fwrule.ipfw_option_icmptypes)
	{
		checkbox_icmptypes->setChecked(TRUE);
		lineedit_icmptypes->setText(fwrule.ipfw_option_icmptypes_string);
	}
	
	if (fwrule.ipfw_option_ipoptions)
	{
		checkbox_ipoptions->setChecked(TRUE);
		lineedit_ipoptions->setText(fwrule.ipfw_option_ipoptions_string);
	}
	
	if (fwrule.ipfw_option_tcpflags)
	{
		checkbox_tcpflags->setChecked(TRUE);
		lineedit_tcpflags->setText(fwrule.ipfw_option_tcpflags_string);
	}
	
	if (fwrule.ipfw_option_tcpoptions)
	{
		checkbox_tcpoptions->setChecked(TRUE);
		lineedit_tcpoptions->setText(fwrule.ipfw_option_tcpoptions_string);
	}
	
	for (i = 0;i<combobox_action->count();i++)
		if (combobox_action->text(i) == fwrule.ipfw_action)
		{
			combobox_action->setCurrentItem(i);
			break;
		}
	if (i == combobox_action->count())
	{
		combobox_action->insertItem(fwrule.ipfw_action,0);
		combobox_action->setCurrentItem(0);
	}
	if (fwrule.ipfw_action == "unreach")
	{
		char tstr[10];
		sprintf(tstr,"%d",(int)fwrule.ipfw_unreach_code);
		combobox_unreach_code->insertItem(tstr,0);
		combobox_unreach_code->setCurrentItem(0);
	}
	
	if (fwrule.ipfw_action == "skipto")
	{
		spinbox_skipto->setValue(fwrule.ipfw_skipto_rule);
	}
	if ((fwrule.ipfw_action =="tee")||(fwrule.ipfw_action =="divert"))
	{
		combobox_teeport->insertItem(fwrule.ipfw_divert_port_name,0);
		combobox_teeport->setCurrentItem(0);
	}
	
	if ((fwrule.ipfw_action =="pipe")||(fwrule.ipfw_action =="queue"))
	{
		combobox_teeport->insertItem(numtostr(fwrule.ipfw_divert_port),0);
		combobox_teeport->setCurrentItem(0);
	}
		
	if (fwrule.ipfw_fwd_address!="")
	{
		lineedit_fwd_address->setText(fwrule.ipfw_fwd_address);
	}
	
	// Source address
	if (fwrule.ipfw_src_address == "any")
		radiobutton_src_any->setChecked(TRUE);
	else
	if (fwrule.ipfw_src_address == "me")
		radiobutton_src_me->setChecked(TRUE);
	else
	{
		for (i = 0;i<combobox_src_address->count();i++)
			if (combobox_src_address->text(i) == fwrule.ipfw_src_address)
			{
				combobox_src_address->setCurrentItem(i);
				break;
			}
		if (i == combobox_src_address->count())
		{
			combobox_src_address->insertItem(fwrule.ipfw_src_address,0);
			combobox_src_address->setCurrentItem(0);
		}
		radiobutton_src_other->setChecked(TRUE);
	}
	if (fwrule.ipfw_not_src)
		checkbox_src_not->setChecked(TRUE);
	if (fwrule.ipfw_src_port != "")
	{
		checkbox_src_port->setChecked(TRUE);
		if (fwrule.ipfw_src_port.find('-') > 0)
		{
			checkbox_src_portrange->setChecked(TRUE);
			spinbox_src_port->setValue(atoi(fwrule.ipfw_src_port.left(fwrule.ipfw_src_port.find('-'))));
			spinbox_src_portrange->setValue(atoi(fwrule.ipfw_src_port.right(fwrule.ipfw_src_port.length()-fwrule.ipfw_src_port.find('-')-1)));
		} else
		if (fwrule.ipfw_src_port.find(',') > 0)
		{
			checkbox_src_portlist->setChecked(TRUE);
			checkbox_src_port->setChecked(FALSE);
			lineedit_src_portlist->setText(fwrule.ipfw_src_port);
		} else
		{
			spinbox_src_port->setValue(atoi(fwrule.ipfw_src_port));
		}
	}
	//**************
	//Destination address
	if (fwrule.ipfw_dst_address == "any")
		radiobutton_dst_any->setChecked(TRUE);
	else
	if (fwrule.ipfw_dst_address == "me")
		radiobutton_dst_me->setChecked(TRUE);
	else
	{
		for (i = 0;i<combobox_dst_address->count();i++)
			if (combobox_dst_address->text(i) == fwrule.ipfw_dst_address)
			{
				combobox_dst_address->setCurrentItem(i);
				break;
			}
		if (i == combobox_dst_address->count())
		{
			combobox_dst_address->insertItem(fwrule.ipfw_dst_address,0);
			combobox_dst_address->setCurrentItem(0);
		}
		radiobutton_dst_other->setChecked(TRUE);
	}
	if (fwrule.ipfw_not_dst)
		checkbox_dst_not->setChecked(TRUE);
	if (fwrule.ipfw_dst_port != "")
	{
		checkbox_dst_port->setChecked(TRUE);
		if (fwrule.ipfw_dst_port.find('-') > 0)
		{
			checkbox_dst_portrange->setChecked(TRUE);
			spinbox_dst_port->setValue(atoi(fwrule.ipfw_dst_port.left(fwrule.ipfw_dst_port.find('-'))));
			spinbox_dst_portrange->setValue(atoi(fwrule.ipfw_dst_port.right(fwrule.ipfw_dst_port.length()-fwrule.ipfw_src_port.find('-')-1)));

		} else
		if (fwrule.ipfw_dst_port.find(',') > 0)
		{
			checkbox_dst_portlist->setChecked(TRUE);
			checkbox_dst_port->setChecked(FALSE);
			lineedit_dst_portlist->setText(fwrule.ipfw_dst_port);
		} else
		{
			spinbox_dst_port->setValue(atoi(fwrule.ipfw_dst_port));
		}
	}
}

void MyRuleEditDialog::updateAll()
{
	updateElements();
	updateRule();
}

void MyRuleEditDialog::updateElements()
{
	spinbox_probability->setEnabled(checkbox_probability->isChecked());
	checkbox_logamount->setEnabled(checkbox_log->isChecked());
	spinbox_logamount->setEnabled(checkbox_log->isChecked() && checkbox_logamount->isChecked());
	combobox_via->setEnabled(checkbox_via->isChecked());
	combobox_uid->setEnabled(checkbox_uid->isChecked());
	combobox_gid->setEnabled(checkbox_gid->isChecked());
	
	lineedit_icmptypes->setEnabled(checkbox_icmptypes->isChecked());
	pushbutton_icmptypes_detailed->setEnabled(checkbox_icmptypes->isChecked());
	lineedit_ipoptions->setEnabled(checkbox_ipoptions->isChecked());
	pushbutton_ipoptions_detailed->setEnabled(checkbox_ipoptions->isChecked());
	lineedit_tcpflags->setEnabled(checkbox_tcpflags->isChecked());
	pushbutton_tcpflags_detailed->setEnabled(checkbox_tcpflags->isChecked());
	lineedit_tcpoptions->setEnabled(checkbox_tcpoptions->isChecked());
	pushbutton_tcpoptions_detailed->setEnabled(checkbox_tcpoptions->isChecked());
	combobox_unreach_code->setEnabled(combobox_action->currentText() == "unreach");
	spinbox_skipto->setEnabled(combobox_action->currentText() == "skipto");
	combobox_teeport->setEnabled((combobox_action->currentText() == "tee")||(combobox_action->currentText() == "divert")||(combobox_action->currentText() == "pipe")||(combobox_action->currentText() == "queue"));
	lineedit_fwd_address->setEnabled(combobox_action->currentText() == "fwd");
	
	
	groupbox_src_address->setEnabled(radiobutton_src_other->isChecked());
	spinbox_src_port->setEnabled(checkbox_src_port->isChecked());
	checkbox_src_portrange->setEnabled(checkbox_src_port->isChecked());
	spinbox_src_portrange->setEnabled(checkbox_src_portrange->isChecked()&&checkbox_src_port->isChecked());
	checkbox_src_portlist->setEnabled(!checkbox_src_port->isChecked());
	lineedit_src_portlist->setEnabled(checkbox_src_portlist->isChecked() && (!checkbox_src_port->isChecked()));
	pushbutton_src_portlist->setEnabled(checkbox_src_portlist->isEnabled());
	
	groupbox_dst_address->setEnabled(radiobutton_dst_other->isChecked());
	spinbox_dst_port->setEnabled(checkbox_dst_port->isChecked());
	checkbox_dst_portrange->setEnabled(checkbox_dst_port->isChecked());
	spinbox_dst_portrange->setEnabled(checkbox_dst_portrange->isChecked()&&checkbox_dst_port->isChecked());
	checkbox_dst_portlist->setEnabled(!checkbox_dst_port->isChecked());
	lineedit_dst_portlist->setEnabled(checkbox_dst_portlist->isChecked() && (!checkbox_dst_port->isChecked()));
	pushbutton_dst_portlist->setEnabled(checkbox_dst_portlist->isEnabled());
	
	
	groupbox_uid->setEnabled((combobox_proto->currentText() == "tcp")||(combobox_proto->currentText() == "udp"));
	groupbox_gid->setEnabled((combobox_proto->currentText() == "tcp")||(combobox_proto->currentText() == "udp"));
	checkbox_setup->setEnabled(combobox_proto->currentText() == "tcp");
	checkbox_established->setEnabled(combobox_proto->currentText() == "tcp");
	groupbox_icmptypes->setEnabled(combobox_proto->currentText() == "icmp");
	groupbox_ipoptions->setEnabled(TRUE/*(combobox_proto->currentText() == "ip")||(combobox_proto->currentText() == "tcp")*/);
	groupbox_tcpflags->setEnabled(combobox_proto->currentText() == "tcp");
	groupbox_tcpoptions->setEnabled(combobox_proto->currentText() == "tcp");
	buttongroup_src_portinfo->setEnabled(combobox_proto->currentText() == "tcp");
	buttongroup_dst_portinfo->setEnabled((combobox_proto->currentText() == "tcp")||(combobox_proto->currentText() == "udp"));
	
	pushbutton_dst_portlist->setEnabled(lineedit_dst_portlist->isEnabled());
	pushbutton_src_portlist->setEnabled(lineedit_src_portlist->isEnabled());
	
}

void MyRuleEditDialog::updateRule()
{
	fwrule.ipfw_number = spinbox_rule_number->value();
	fwrule.ipfw_bprob = checkbox_probability->isChecked();
	fwrule.ipfw_matchprob = ((double)spinbox_probability->value())/100;
	fwrule.ipfw_blog = checkbox_log->isChecked() && checkbox_log->isEnabled();
	fwrule.ipfw_blogamount = checkbox_logamount->isChecked() && checkbox_logamount->isEnabled();
	fwrule.ipfw_logamount = spinbox_logamount->value();
	fwrule.ipfw_not_src = checkbox_src_not->isChecked() && checkbox_src_not->isEnabled();
	fwrule.ipfw_not_dst = checkbox_dst_not->isChecked() &&	checkbox_dst_not->isEnabled();
	fwrule.ipfw_action = combobox_action->currentText();
	fwrule.ipfw_protocol = combobox_proto->currentText();
	fwrule.ipfw_divert_port = 0;
	fwrule.ipfw_divert_port_name = combobox_teeport->currentText();
	fwrule.ipfw_unreach_code = atoi(combobox_unreach_code->currentText());
	fwrule.ipfw_fwd_address = lineedit_fwd_address->text();
	fwrule.ipfw_skipto_rule = spinbox_skipto->value();
	
	if (radiobutton_src_me->isChecked())
		fwrule.ipfw_src_address = "me";
	else if(radiobutton_src_any->isChecked())
		fwrule.ipfw_src_address = "any";
	else fwrule.ipfw_src_address = combobox_src_address->currentText();
	if (fwrule.ipfw_src_address == "")
		fwrule.ipfw_src_address = "any";
	fwrule.ipfw_src_port = "";
	if (checkbox_src_port->isChecked()&&checkbox_src_port->isEnabled())
	{
		fwrule.ipfw_src_port = spinbox_src_port->text();
		if (checkbox_src_portrange->isChecked()&&checkbox_src_portrange->isEnabled())
		{
			fwrule.ipfw_src_port+="-";
			fwrule.ipfw_src_port+=spinbox_src_portrange->text();
		}
	}
	else if (checkbox_src_portlist->isChecked()&&checkbox_src_portlist->isEnabled())
	{
		fwrule.ipfw_src_port = lineedit_src_portlist->text();
	}

	if (radiobutton_dst_me->isChecked())
		fwrule.ipfw_dst_address = "me";
	else if(radiobutton_dst_any->isChecked())
		fwrule.ipfw_dst_address = "any";
	else fwrule.ipfw_dst_address = combobox_dst_address->currentText();
	if (fwrule.ipfw_dst_address == "")
		fwrule.ipfw_dst_address = "any";
	fwrule.ipfw_dst_port = "";
	if (checkbox_dst_port->isChecked()&&checkbox_dst_port->isEnabled())
	{
		fwrule.ipfw_dst_port = spinbox_dst_port->text();
		if (checkbox_dst_portrange->isChecked()&&checkbox_dst_portrange->isEnabled())
		{
			fwrule.ipfw_dst_port+="-";
			fwrule.ipfw_dst_port+=spinbox_dst_portrange->text();
		}
	}
	else if (checkbox_dst_portlist->isChecked()&&checkbox_dst_portlist->isEnabled())
	{
		fwrule.ipfw_dst_port = lineedit_dst_portlist->text();
	}

	fwrule.ipfw_uid = (checkbox_uid->isChecked()&&checkbox_uid->isEnabled()) ? combobox_uid->currentText() : QString("") ;
	fwrule.ipfw_gid = (checkbox_gid->isChecked()&&checkbox_gid->isEnabled()) ? combobox_gid->currentText() : QString("") ;
	if (radiobutton_incoming->isChecked())
		fwrule.ipfw_direction = "in";
	else if (radiobutton_outcoming->isChecked())
		fwrule.ipfw_direction = "out";
	else fwrule.ipfw_direction = "";
	fwrule.ipfw_option_keepstate = checkbox_keepstate->isChecked()&&checkbox_keepstate->isEnabled();
	fwrule.ipfw_option_keepstate_method = "";
	fwrule.ipfw_option_bridged = checkbox_bridged->isChecked()&&checkbox_bridged->isEnabled();
	fwrule.ipfw_option_fragment = checkbox_fragment->isChecked()&&checkbox_fragment->isEnabled();
	fwrule.ipfw_option_established = checkbox_established->isChecked()&&checkbox_established->isEnabled();
	fwrule.ipfw_option_setup = checkbox_setup->isChecked()&&checkbox_setup->isEnabled();
	fwrule.ipfw_option_tcpflags = checkbox_tcpflags->isChecked()&&checkbox_tcpflags->isEnabled()&&(lineedit_tcpflags->text()!="");
	fwrule.ipfw_option_tcpflags_string = lineedit_tcpflags->text();
	fwrule.ipfw_option_ipoptions = checkbox_ipoptions->isChecked()&&checkbox_ipoptions->isEnabled()&&(lineedit_ipoptions->text()!="");
	fwrule.ipfw_option_ipoptions_string = lineedit_ipoptions->text();
	fwrule.ipfw_via = (checkbox_via->isChecked()) ? combobox_via->currentText() : QString("");
	fwrule.ipfw_option_tcpoptions = checkbox_tcpoptions->isChecked()&&checkbox_tcpoptions->isEnabled()&&(lineedit_tcpoptions->text()!="");
	fwrule.ipfw_option_tcpoptions_string = lineedit_tcpoptions->text();
	fwrule.ipfw_option_icmptypes = checkbox_icmptypes->isChecked()&&checkbox_icmptypes->isEnabled()&&(lineedit_icmptypes->text()!="");
	fwrule.ipfw_option_icmptypes_string = lineedit_icmptypes->text();
	
	strRule = fwrule.formRule();
	lineedit_rule->setText(strRule);
}

void MyRuleEditDialog::needUpdate()
{
	if (bNoUpdateNow) return;
	updateAll();
}
void MyRuleEditDialog::editIcmpTypes()
{
	IcmptypeListDialog dlg_icmptypes(_icmpcodes,16,lineedit_icmptypes->text(),this);
	if (dlg_icmptypes.exec())
		lineedit_icmptypes->setText(dlg_icmptypes.strResult);
}

void MyRuleEditDialog::editIpOptions()
{
	OptionListDialog dlg_ipoptions(IPOPTION_LIST,IPOPTION_LIST_SIZE,lineedit_ipoptions->text(),this);
	if (dlg_ipoptions.exec())
		lineedit_ipoptions->setText(dlg_ipoptions.strResult);
}

void MyRuleEditDialog::editSrcPortList()
{
	IcmptypeListDialog dlg_srcportlist(SERVICE_LIST,SERVICE_LIST_SIZE,lineedit_src_portlist->text(),this);
	if (dlg_srcportlist.exec())
		lineedit_src_portlist->setText(dlg_srcportlist.strResult);
}

void MyRuleEditDialog::editTcpFlags()
{
	OptionListDialog dlg_tcpflags(TCPFLAG_LIST,TCPFLAG_LIST_SIZE,lineedit_tcpflags->text(),this);
	if (dlg_tcpflags.exec())
		lineedit_tcpflags->setText(dlg_tcpflags.strResult);
}

void MyRuleEditDialog::editTcpOptions()
{
	OptionListDialog dlg_tcpoptions(TCPOPTION_LIST,TCPOPTION_LIST_SIZE,lineedit_tcpoptions->text(),this);
	if (dlg_tcpoptions.exec())
		lineedit_tcpoptions->setText(dlg_tcpoptions.strResult);
}

void MyRuleEditDialog::editDstPortList()
{
	IcmptypeListDialog dlg_dstportlist(SERVICE_LIST,SERVICE_LIST_SIZE,lineedit_dst_portlist->text(),this);
	if (dlg_dstportlist.exec())
		lineedit_dst_portlist->setText(dlg_dstportlist.strResult);
}

void MyRuleEditDialog::onOK()
{
	if (combobox_src_address->isEnabled() && (combobox_src_address->currentText()!=""))
	{
		if (!app_rc.hostnames.contains(combobox_src_address->currentText()))
			app_rc.hostnames.prepend(combobox_src_address->currentText());
	}
	if (combobox_dst_address->isEnabled() && (combobox_dst_address->currentText()!=""))
		if (combobox_src_address->isEnabled() &&(combobox_src_address->currentText()!=combobox_dst_address->currentText()))
		{
			if (!app_rc.hostnames.contains(combobox_dst_address->currentText()))
				app_rc.hostnames.prepend(combobox_dst_address->currentText());
		}
	accept();
}

