#include "comtypes.H"
#include "allglobals.H"
#include "scriptdialog.H"
#include "mainform.H"
#include "kernelconfigtab.H"
#include <qlabel.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qmultilineedit.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qfont.h>
#include <qlistbox.h>
#include <fstream.h>
#include <unistd.h>
#include <sys/types.h>
/* 
 *  Constructs a ScriptDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
ScriptDialog::ScriptDialog( QWidget* parent,  const char* name, WFlags fl )
    : ScriptDialogBase( parent, name, fl )
{
	checkbox_sysctl->setChecked(FALSE);
	checkbox_current->setChecked(FALSE);
	checkbox_repository->setChecked(TRUE);
	checkbox_sysctl->setEnabled(FALSE);
	checkbox_current->setEnabled(FALSE);
	checkbox_repository->setEnabled(TRUE);

	if (!getuid() || bUseSudo)
	{
		checkbox_sysctl->setEnabled(TRUE);
		checkbox_current->setEnabled(TRUE);
		checkbox_current->setChecked(TRUE);
	}

	multilineedit_preview->setFont(QFont("Fixed"));

	connect(pushbutton_create,SIGNAL(clicked()),this,SLOT(writeScript()));
	connect(pushbutton_save,SIGNAL(clicked()),this,SLOT(saveScript()));
	
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ScriptDialog::~ScriptDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

void ScriptDialog::writeScript()
{
	multilineedit_preview->clear();
	multilineedit_preview->insertLine("#!/bin/sh");
	multilineedit_preview->insertLine("#\tFirewall script - generated by Qtfw;");
	multilineedit_preview->insertLine("#\tYou may insert this into rc.firewall");
	if (checkbox_sysctl->isChecked())
	{
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.verbose=")+(kerneltab->checkbox_fw_verbose->isChecked() ? "1" : "0"));
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.debug=")+(kerneltab->checkbox_fw_debug->isChecked() ? "1" : "0"));
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.enable=")+(kerneltab->checkbox_fw_enable->isChecked() ? "1" : "0"));
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.verbose_limit=")+kerneltab->spinbox_fw_verbose_limit->text());
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.dyn_max=")+kerneltab->spinbox_fw_dyn_max->text());
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.dyn_ack_lifetime=")+kerneltab->spinbox_fw_ack_lifetime->text());
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.dyn_syn_lifetime=")+kerneltab->spinbox_fw_syn_lifetime->text());
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.dyn_fin_lifetime=")+kerneltab->spinbox_fw_fin_lifetime->text());
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.dyn_rst_lifetime=")+kerneltab->spinbox_fw_rst_lifetime->text());
		multilineedit_preview->insertLine(QString("sysctl -w net.inet.ip.fw.dyn_short_lifetime=")+kerneltab->spinbox_fw_short_lifetime->text());
	}	
	if (checkbox_current->isChecked())
	{
		QListViewItem * listitem = mainform->qlistview_rules->firstChild();
		if (listitem)
			multilineedit_preview->insertLine(QString("ipfw add ")+listitem->text(2));
		else return;
		while ((listitem = listitem->nextSibling()) != NULL)
			multilineedit_preview->insertLine(QString("ipfw add ")+listitem->text(2));
	}
	if (checkbox_repository->isChecked())
	{
		for (unsigned int i = 0; i<mainform->listbox_repository->count();i++)
		{
			multilineedit_preview->insertLine(QString("ipfw add ")+mainform->listbox_repository->text(i));
		}
	}
}

void ScriptDialog::saveScript()
{
	if (multilineedit_preview->text().length()<3)
		return;
	QFileDialog fdlg(".",QString::null,this,"Save rules to file:",TRUE);
	fdlg.setMode(QFileDialog::AnyFile);
	if (fdlg.exec())
	{
		if (fdlg.selectedFile().isNull())
			return;
		ofstream of_script(fdlg.selectedFile(),ios::out|ios::trunc);
		if (!of_script.good())
		{
			QMessageBox::critical(this,"Error!","Error opening file!");
			return;
		}
		of_script <<multilineedit_preview->text() <<endl;
	}	
}

